import { REST } from "@discordjs/rest"
import { WebSocketManager } from "@discordjs/ws"
import { Client, GatewayIntentBits } from "@discordjs/core"
import { config } from "./config.js"
import { VoiceStateManager } from "./utils/wrapper/VoiceState.js"
import { RecordingSessionManager } from "./utils/sessions.js"
import { PrismaClient } from "@prisma/client"

export const rest = new REST({ version: "10" }).setToken(config.token)

export const gateway = new WebSocketManager({
	token: config.token,
	intents: GatewayIntentBits.Guilds | GatewayIntentBits.GuildVoiceStates,
	rest,
	shardIds: [0],
	shardCount: 1,
})

export const client = new Client({ rest, gateway })

export const voiceStateManager = new VoiceStateManager()

export const sessionManager = new RecordingSessionManager()

export const db = new PrismaClient()
