import path from "path"
import { db } from "../api.js"
import { spawn } from "child_process"
import { existsSync, mkdirSync } from "fs"

export const mergeAudio = async (
	id: string,
	callback?: (code: number | null) => void
) => {
	const finalDir = "recordings/final"
	const sourceDir = path.resolve("recordings", id)

	if (!existsSync(finalDir)) {
		mkdirSync(finalDir, { recursive: true })
	}

	const files = await db.file.findMany({
		where: {
			session: {
				id,
			},
		},
	})

	let inputs: string[] = []

	let filter: string[] = []
	let trackNames: string[] = []

	let counter = 0

	for (const file of files) {
		const filePath = path.join(sourceDir, file.filename)
		const index = counter++
		const trackName = `[c${index}]`

		inputs.push("-i", filePath)

		filter.push(
			`[${index}:a] adelay=${file.timestamp}|${file.timestamp} ${trackName}`
		)

		trackNames.push(trackName)
	}

	filter.push(
		`${trackNames.join(" ")} amix=inputs=${
			trackNames.length
		}:normalize=0 [out]`
	)

	const args = [
		...inputs,
		"-filter_complex",
		filter.join(";"),
		"-map",
		"[out]",
		"-async",
		"1",
		"-y",
		path.join(finalDir, `${id}.ogg`),
	]

	const p = spawn("ffmpeg", args).on("exit", (code) => {
		console.log("ffmpeg exited with code", code)

		callback?.(code)
	})
	p.stderr.pipe(process.stderr)
}
