import { GatewayVoiceState, Snowflake } from "discord-api-types/v10"
import { Collection } from "@discordjs/collection"
import { sessionManager } from "../../api.js"

type GuildVoiceState = Omit<GatewayVoiceState, "guild_id">

export class VoiceStateManager {
	guilds = new Collection<string, Collection<string, VoiceState>>()

	addGuild(id: Snowflake, states: GuildVoiceState[]) {
		const guild = new Collection<Snowflake, VoiceState>()
		for (const state of states) {
			guild.set(state.user_id, new VoiceState(state, id))
		}
		this.guilds.set(id, guild)
	}

	removeGuild(id: Snowflake) {
		this.guilds.delete(id)
	}

	getUserVC(guild: Snowflake, member: Snowflake): string | null {
		const guildStates = this.guilds.get(guild)
		if (!guildStates) return null
		return guildStates.get(member)?.channelId ?? null
	}
}

export class VoiceState {
	id!: Snowflake
	channelId!: Snowflake | null

	mute!: boolean

	constructor(data: GuildVoiceState, public guild: Snowflake) {
		this.update(data)
	}

	update(data: GuildVoiceState) {
		this.channelId = data.channel_id
		this.id = data.user_id
		this.mute = data.mute || data.self_mute
	}
}
